Program KrsKaoJadro
! test primjer numerickog rjesenja strujanja u krsu
! promatra se sliv bez diskretizacije na podslivove
use ObicneDJ
implicit none
	!
	real A/292561341/	! povrsina sliva
	real L/25000/		! duzina kanala
	real Zizvor/25/		! kota izvora
	real Hpocetno/150/	! pocetna piez. visina
	!
	real DtSat/3600/	! sekunda u satu
	real DtDan/86400/	! sekunda u danu
	! polja za grafove VolKise(t) i Qjadro(t)
	real tKisa(365),VolKisa(365),tJadro(365),Qjadro(365)
	! polja za grafove K(h) i S(h)
	real Zvar(100),KgKanal(100),Sporoznost(100)
	integer nData		!  broj stvarnih podataka
	
	integer korak,nKorak/730/	! brojilo koraka i ukupni broj koraka
	real Dt/43200/		! korak integracije pola dana

	real Qizvor		! izracunati protok izvora
	!varijable za rjesenje jednadzbe
	real t				 ! nezavisna varijabla vrijeme
	real Y(1)			 ! polje za zavisne varijable (rjesenje H)
	real F(1)			 ! polje za desnu stranu jednadzbe

	call Input

	open(unit=1,File="KrsKaoJadro.txt")
	write(1,*) '"Vrijeme" "Q izvor" "Q mjereno"'

	call PocetniUvjeti
	write(1,'(f10.4,2f10.4)') t/86400.0,Qizvor,pgVal(365,tJadro,Qjadro,t)
	! integracija Runge-Kutta postupkom
	if (OpenObicneDJ(1)) then	! jedna diferencijalna jednadzba
		do korak=1,nKorak
			if (RK_Korak(t,Y,Dt,DesnaStrana)) then
				write(1,'(f10.4,2f10.4)') t/86400.0,Qizvor,pgVal(365,tJadro,Qjadro,t)
			endif
		enddo
		call CloseObicneDJ
	endif

	close(1)

contains

	subroutine Input
	implicit none
	integer k
		! ucitavanje podataka o izmjerenom protku izvora Jadra
		open(unit=1,File="Qjadro2001.dat")
			do k=1,365
				read(1,*) tJadro(k),Qjadro(k)
			enddo
		close(1)
		! ucitavanje podataka o paloj kisi
		open(unit=1,File="VolKiseDicmo.dat")
			do k=1,365
				read(1,*) tKisa(k),VolKisa(k)
			enddo
		close(1)
		!ucitavanje podataka o provodljivosti K kanala
		!ucitavanje podataka o poroznosti S sliva
		open(unit=1,File="KS-jadro.dat")
			read(1,*) nData
			do k=1,nData
				read(1,*) Zvar(k),KgKanal(k),Sporoznost(k)
			enddo
		close(1)
	end subroutine Input

	subroutine PocetniUvjeti
	implicit none
	real Kh
		! pridruzivanje pocetnih uvjeta
		t=0
		Y(1)=Hpocetno
		Kh=pgVal(nData,Zvar,KgKanal,Y(1))
		! pocetni protok
		Qizvor=Kh*sign(1.0,Y(1)-Zizvor)*sqrt(abs(Y(1)-Zizvor)/L)
	end subroutine PocetniUvjeti

	! 'call back' procedura
	subroutine DesnaStrana(n,X,Y,F,ierr)
	implicit none
	integer n	! broj jednadzbi
	real X		! vrijednost nezavisne varijable
	real Y(n)	! vrijednosti zavisnih varijabli (nepoznanica)
	real F(n)	! vrijednosti desne strane
	integer ierr	! indikator pogreske u proracunu

	! lokalne varijable
	real pKIsa,vKisa1,vKisa2,Qkisa,Kh,Sh

		! proracun pale kise
		vKisa2=pgVal(365,tKisa,VolKisa,X+DtDan)
		vKisa1=pgVal(365,tKisa,VolKisa,X+DtDan-DtSat)
		pKisa = (vKisa2-vKisa1)/DtSat	! intenzitet p [m/s]
		Qkisa=pKisa*A					! protok kise
		Kh=pgVal(nData,Zvar,KgKanal,Y(1))	! provodljivost kanala
		Sh=pgVal(nData,Zvar,Sporoznost,Y(1))	! poroznost
		Qizvor=Kh*sign(1.0,Y(1)-Zizvor)*sqrt(max(abs(Y(1)-Zizvor)/L,0.))
		F(1)=(Qkisa-Qizvor)/(Sh*A)	! desna strana

	end subroutine DesnaStrana

	real Function PgVal(Ndata, x, y, xval)
	! izracunava vrijednost poligonalnog grafa y(x)
	! za zadanu vrijednost argumenta xval
	! za xval<xmin vraca y(xmin)
	! za xval.xmax vraca y(xmax)
	integer Ndata
	real x(Ndata),y(ndata),xval
	real nagib
	integer i
		If (xval .le. x(1)) Then
			PgVal = y(1)
		ElseIf (xval .gt. x(Ndata)) Then
			PgVal = y(Ndata)
		Else
			do i = 2 , Ndata
				If (xval .eq. x(i)) Then
					PgVal = y(i)
					return
				ElseIf (xval .lt. x(i)) Then
					nagib = (y(i) - y(i - 1)) / (x(i) - x(i - 1))
					PgVal = y(i - 1) + nagib * (xval - x(i - 1))
					return
				End If
			enddo
		End If
	End Function PgVal

end program
