Module Moody
implicit none
!
! modul sadrzi funkcijske potprograme za izracunavanje
! koeficijenta otpora tecenja u cijevima
!
! otpori tecenja u cijevima ovise o Reynoldsovu broju  Re=v*D/ni
! i relativnoj hrapavosti k/D
!
! za Re<2320 tecenje je laminarno, otpori ne ovise o rel. hrap. k/D
!            i ravna se po Hagen-Poisseuilleovom zakonu lambda=64/Re
! za Re>2320 tecenje je turbulentno.
!            Otpori ovise o rezimu strujanja. Postoje tri
!            rezima:
!                   turbulentno hrapavi, ovisi o k, ne ovisi o Re
!                   turbulentno glatki, ne ovisi o k, ovisi o Re
!                   turbulentno prijelazni, ovisi o k i Re
! (Moody-jev dijagram prikazuje otpore tecenja
!            u svim rezimima strujanja)
! 
contains

real function amoody(re,rh)
! koeficijent otpora lambda (Moody-jev dijagram)
! re - reynoldsov broj
! rh - relativna hrapavost
real re,rh
 if(re.le.2320.) then
  amoody=64./(re+1.e-15)
 else
  amoody=tprell(rh,re)
 endif
end function

real function  tprell(rh,re)
! koeficijent otpora lambda (turbulentno prijelazni rezim)
! po Colebrook-Whiteova formula
! re - reynoldsov broj
! rh - relativna hrapavost
real re,rh
real x
integer i
 x=thrapl(rh)
 do i=1,50  ! iteracija
  x=1./(1.74-2.*alog10(2.*rh+18.7/re/sqrt(x)))**2
 enddo
 tprell=x
end function

real function  thrapl(rh)
! koeficijent otpora lambda (turbulentno hrapavi rezim)
! po Prandtl-Karmanovom zakonu
! rh - relativna hrapavost
real rh
 if(rh.eq.0) rh=1e-20
 thrapl=1./(1.14-2.*alog10(rh))**2
end function

End Module Moody
