module trapez

	private Toleranca
	
contains

	subroutine AORB_trapez(B0,m,D,A,O,R,B)
	! geometrijske karakteristike trapeznog presjeka
	implicit none
	real D      ! dubina
	real B0     ! sirina dna
	real m      ! 1:m
	real A      ! povrsina
	real O      ! omoceni opseg
	real R      ! hidraulicki radius
	real B      ! sirina vodnog lica
	real dd
		dd=D
		if(D.lt.0.)  dd=0.
		A=B0*dd+m*dd**2
		O=B0+2*dd*sqrt(1+m**2)
		B=B0+2*m*dd
		if(dd.le.0) then
			R=0
		else
			R=A/O
		endif
	end subroutine AORB_trapez
	!
	real function V_trapez(D,B0,m,h,I0)
	! brzina
	implicit none
	real D		! dubina
	real h		! hrapavost
	real B0		! sirina dna
	real m		! 1:m
	real I0		! pad
	real A,O,R,B
	real vi/1.13e-6/
	real g /9.81/
	real c,rh
		if(D>0.) then
			if(I0>0) then
				call AORB_trapez(B0,m,D,A,O,R,B)
				rh=h/(4*R)
				c=1.14-2*alog10(rh+9.35*vi/(4*R*sqrt(8*g*R*I0)))
				V_trapez=c*sqrt(8*g*R*I0)
			else
				V_trapez=0
			endif
		else
			V_trapez=0
		endif
		return
	end function V_trapez
	!
	real function Ie_trapez(Q,D,B0,m,h)
	! energijski pad
	implicit none
	real Q		! protok
	real D		! dubina
	real h		! hrapavost
	real B0		! sirina dna
	real m		! 1:m
	real Ie		! pad energijske linije
	real A,O,R,B
	real vi/1.13e-6/
	real g /9.81/
	real c,rh,V
	integer k
		if(D>0.) then
			call AORB_trapez(B0,m,D,A,O,R,B)
			V=Q/A
			rh=h/(4*R)
			c=1.14-2*alog10(rh)	! turb. hrapavo strujanje
			Ie=abs(v)*v/(8*g*R)/c**2
			! popravak za turb. prijelazno strujanje u 10 iteracija
			do k=1,10
				c=1.14-2*alog10(rh+9.35*vi/(4*R*sqrt(8*g*R*Ie)))
				Ie=abs(v)*v/(8*g*R)/c**2
			enddo
			Ie_trapez=Ie
		else
			Ie_trapez=0
		endif
		return
	end function Ie_trapez
	!
	real function QN_trapez(D,B0,m,h,I0)
	! protok za normalnu dubinu u jednolikom strujanju
	implicit none
	real D		! dubina
	real B0		! sirina dna
	real m		! 1:m
	real h		! apsolutna hrapavost
	real I0		! pad
	real A,O,R,B
		if(D.gt.0.) then
			call AORB_trapez(B0,m,D,A,O,R,B)
			QN_trapez=A*V_trapez(D,B0,m,h,I0)
		else
			QN_trapez=0
		endif
		return
	end function QN_trapez
	!
	real function DN_trapez(Q0,h,I0,B0,m,nZnam,ierr)
	! normalna dubina
	implicit none
	real Q0		! protok
	real h		! apsolutna hrapavost
	real I0		! pad
	real B0		! sirina dna
	real m		! 1:m
	integer nZnam
	integer ierr	! indikator greske: 0-nema 1 - greska
	integer k
	integer mxk /100/
	real D,dp,dk,res
	real epsQ		! tocnost iteracije
	!real QN_trapez
		dp=0.		! predpostvlja se da ce normalna dubina
		dk=1000.	! biti izmedju 0 i 1000 metara
		D = dp
		res = QN_trapez(D,B0,m,h,I0) - Q0
		k=0
		epsQ=Toleranca(Q0,nZnam)
		do while(abs(res).gt.epsQ.and.k.lt.mxk)
			D = 0.5*(dp+dk)
			res = QN_trapez(D,B0,m,h,I0) - Q0
			k=k+1
			if(res.gt.0.) then
				dk = D
			elseif(res.lt.0.) then
				dp=D
			else
				exit
			endif
		enddo
		DN_trapez=D
		if(k.ge.mxk) then
			! iteracija nije konvergirala u mxk koraka
			ierr=1
		else
			ierr=0
		endif
		return
	end function DN_trapez
	!
	real function HS_trapez(D,B0,m,Q0,Beta,ierr)
	! specificna energija
	implicit none
	real D		! dubina
	real B0		! sirina dna
	real m		! 1:m
	real Q0		! protok
	real Beta	! kut osi strujanja prema horizontali
	integer ierr	! indikator greske: 0-nema 1 - greska
	real g/9.81/
	real A,O,R,B
		if(D.gt.0.) then
			call AORB_trapez(B0,m,D,A,O,R,B)
			HS_trapez=D*cos(Beta)+Q0**2/(2*g*A**2)
			ierr=0
		else
			HS_trapez=0
			ierr=1	! 'HS_trapez -> inf'
		endif
		return
	end function HS_trapez
	!
	real function QC_trapez(D,B0,m,Beta,ierr)
	! kriticni protok
	implicit none
	real D		! dubina
	real B0		! sirina dna
	real m		! 1:m
	real Beta	! kut osi strujanja prema horizontali
	integer ierr	! indikator greske: 0-nema 1 - greska
	real A,O,R,B
	real g/9.81/
		call AORB_trapez(B0,m,D,A,O,R,B)
		if(B.gt.0.) then
			QC_trapez=A*sqrt(cos(Beta)*g*A/B)
			ierr=0
		else
			QC_trapez=0
			ierr=1	! 'QC_trapez -> inf'
		endif
		return
	end function QC_trapez
	!
	real function DC_trapez(Q0,B0,m,Beta,nZnam,ierr)
	! kriticna dubina
	implicit none
	real Q0		! protok
	real B0		! sirina dna
	real m		! 1:m
	real Beta	! kut osi strujanja prema horizontali
	integer nZnam	! tocnost na broj znamenaka
	integer ierr	! indikator greske: 0-nema 1 - greska
	real epsQ		! tocnost iteracije
	integer k
	integer mxk /100/
	real D,dp,dk,res
	!real QC_trapez
		dp=0.		! granica za res < 0.
		dk=100  !0.99*fi		! granica za res > 0.
		D = dk
		res=QC_trapez(D,B0,m,Beta,ierr)-Q0
		if(ierr.ne.0) return
		k=0
		epsQ=Toleranca(Q0,nZnam)
		do while(abs(res).gt.epsQ.and.k.lt.mxk)
			D = 0.5*(dp+dk)
			res=QC_trapez(D,B0,m,Beta,ierr)-Q0
			if(ierr.ne.0) return
			k=k+1
			if(res.gt.0.) then
				dk = D
			elseif(res.eq.0.) then
				exit
			else
				dp=D
			endif
		enddo
		DC_trapez=D
		if(k.ge.mxk) then
			! iteracija nije konvergirala u mxk koraka
			ierr=1
		else
			ierr=0
		endif
		return
	end function DC_trapez
	!
	real function DM_trapez(Q0,Hs,B0,m,Beta,nZnam,ierr)
	! dubina u mirnom strujanju
	implicit none
	real Q0		! protok
	real Hs		! specificna energija
	real B0		! sirina dna
	real m		! 1:m
	real Beta	! kut osi strujanja prema horizontali
	integer nZnam	! tocnost na broj znamenaka
	integer ierr	! indikator greske: 0-nema 1 - greska
	real epsH		! tocnost Hs
	integer k
	integer mxk /100/
	real D,dp,dk,res
	!real DC_trapez,HS_trapez
		dp=100      !
		dk=DC_trapez(Q0,B0,m,Beta,nZnam,ierr)
		if(ierr.ne.0) return
		D = dk
		res = Hs - HS_trapez(D,B0,m,Q0,Beta,ierr)
		if(ierr.ne.0) return
		k=0
		epsH=Toleranca(Hs,nZnam)
		do while(abs(res).gt.epsH.and.k.lt.mxk)
			D = 0.5*(dp+dk)
			res = Hs - HS_trapez(D,B0,m,Q0,Beta,ierr)
			if(ierr.ne.0) return
			k=k+1
			if(res.gt.0.) then
				dk = D
			elseif(res.eq.0.) then
				exit
			else
				dp=D
			endif
		enddo
		DM_trapez=D
		if(k.ge.mxk) then
			! iteracija nije konvergirala u mxk koraka
			! vjerojatno nema rjeenja.
			! Uzima se kriticna dubina kao rjesenje
			DM_trapez=DC_trapez(Q0,B0,m,Beta,nZnam,ierr)
			ierr=-1
		else
			ierr=0
		endif
		return
	end function DM_trapez
	!
	real function DS_trapez(Q0,Hs,B0,m,Beta,nZnam,ierr)
	! dubina u silovitom strujanju
	implicit none
	real Q0		! protok
	real Hs		! specificna energija
	real B0		! sirina dna
	real m		! 1:m
	real Beta	! kut osi strujanja prema horizontali
	integer nZnam	! tocnost na broj znamenaka
	integer ierr	! indikator greske: 0-nema 1 - greska
	real epsH		! tocnost Hs
	integer k
	integer mxk /100/
	real D,dp,dk,res
	!real DC_trapez,HS_trapez
		dp=0.
		dk=DC_trapez(Q0,B0,m,Beta,nZnam,ierr)
		if(ierr.ne.0) return
		D = dk
		res = Hs - HS_trapez(D,B0,m,Q0,Beta,ierr)
		if(ierr.ne.0) return
		k=0
		epsH=Toleranca(Hs,nZnam)
		do while(abs(res).gt.epsH.and.k.lt.mxk)
			D = 0.5*(dp+dk)
			res = Hs - HS_trapez(D,B0,m,Q0,Beta,ierr)
			if(ierr.ne.0) return
			k=k+1
			if(res.gt.0.) then
				dk = D
			elseif(res.eq.0.) then
				exit
			else
				dp=D
			endif
		enddo
		DS_trapez=D
		if(k.ge.mxk) then
			! iteracija nije konvergirala u mxk koraka
			! vjerojatno nema rjeenja.
			! Uzima se kriticna dubina kao rjesenje
			DS_trapez=DC_trapez(Q0,B0,m,Beta,nZnam,ierr)
			ierr=-1
		else
			ierr=0
		endif
		return
	end function DS_trapez
	!
	real function IC_trapez(Q0,B0,m,h,nZnam,ierr)
	! kriticni pad
	implicit none
	real Q0		! protok
	real B0		! sirina dna
	real m		! 1:m
	real h		! hrapavost
	integer nZnam	! broj tocnih znamenaka
	integer ierr	! indikator greske: 0-nema 1 - greska
	real epsV
	real Dc,Vc,Ac,Bc,Oc,Rc
	integer k
	integer mxk /100/
	real vi/1.13e-6/
	real g /9.81/
	real res,Ip,Ik,Ic
	!real DC_trapez,V_trapez
		Dc = DC_trapez(Q0,B0,m,0.,nZnam,ierr)
		if(ierr.ne.0) return
		call AORB_trapez(B0,m,Dc,Ac,Oc,Rc,Bc)
		Vc=Q0/Ac
		Ip=0.	! predpostvlja se da ce kriticni pad
		Ik=1.	! biti izmedju 0 i 1000 promila
		Ic = Ip
		res = V_trapez(Dc,B0,m,h,Ic)-Vc
		k=0
		epsV=Toleranca(Vc,nZnam)
		do while(abs(res).gt.epsV.and.k.lt.mxk)
			Ic = 0.5*(Ip+Ik)
			res = V_trapez(Dc,B0,m,h,Ic)-Vc
			k=k+1
			if(res.gt.0.) then
				Ik = Ic
			elseif(res.lt.0.) then
				Ip=Ic
			else
				exit
			endif
		enddo
		IC_trapez=Ic
		if(k.ge.mxk) then
			! iteracija nije konvergirala u mxk koraka
			ierr=1
		else
			ierr=0
		endif
	end function IC_trapez
	!
	real function Manning_trapez(R,h,Ie)
	! Manningov koeficijent
	implicit none
	real R		! hidraulicki radius
	real h		! hrapavost
	real Ie		! pad energijske linije
	real vi/1.13e-6/
	real g /9.81/
	real c,rh
	integer k
		rh=h/(4*R)
		if(Ie.eq.0.) then	! turbulentno hrapavo strujanje
			c=1.14-2*alog10(rh)
		else	! turbulentno prijelazno strujanje
			c=1.14-2*alog10(rh+9.35*vi/(4*R*sqrt(8*g*R*Ie)))
		endif
		Manning_trapez=R**(1./6.)/sqrt(8*g)/c
		return
	end function Manning_trapez
	!
	! privatna funkcija, vidljiva samo u ovom modulu
	! izracunavanje tolerancije za real X
	! na zadani broj znacajnih znamenaka
	real function Toleranca(x,nZnamenaka)
	implicit none
	real x
	integer nZnamenaka
		if (x.eq.0) then
			Toleranca=5*10**(-nZnamenaka-1)
		elseif (abs(x).gt.1) then
			Toleranca=5 * 10**(aint(Log10(Abs(x)))-nZnamenaka)
		elseif (abs(x).lt.1) then
			Toleranca=5 * 10**(aint(Log10(Abs(x)))-nZnamenaka-1)
		endif
	end function Toleranca

end module trapez
