Program PotencijalnoStrujanje

!   skolski primjer rjesenja Laplaceove jednadzbe
!   metodom konacnih elemenata
!
!   napisao:
!             Prof. dr. Vinko Jovic
!

! parametri dimenzija modela
integer Ncvor	! broj cvorova
integer Nelem	! broj elemenata
integer nStup	! broj stupaca (dijagonala) u matrici u stednom obliku
integer gDiag	! pozicija u glavne dijagonale u stednom obliku matrice
real,allocatable:: Xc(:)	! alokatibilno polje X koordinata
real,allocatable:: Yc(:)	! alokatibilno polje Y koordinata
integer,allocatable:: eVeze(:,:)	! alokatibilno polje tablicu elementnih veza
!	TipRuvjet(cvor)=0	-	obicni cvor (zbroj tokova jednak nuli)
!	TipRuvjet(cvor)=1	-	zadana vrijednost rjesenja
integer,allocatable:: TipRuvjet(:)	! alokatibilno polje tipa rubnog uvjeta
real,allocatable:: RubniUvjet(:)	! alokatibilno polje vrijednosti rubnog uvjeta

real,allocatable:: Aglob(:,:)	! prostor za globalnu matricu
real,allocatable:: Bglob(:)		! prostor za globalni vektor

character*(256) JobFileName

	write(*,'(A\)') 'LAPLACE: Utipkaj ime ulaznog filea:  '
	read(*,*) JobFileName

	call Input(JobFileName)

	gDiag = SirinaPojasa()+1
	nStup=2* gDiag -1

	call SlaganjeGlobalneMatrice
	call RubniUvjeti
	call PojasniSustav(Ncvor,nStup,Aglob,Bglob)
	call Ispis(trim(JobFileName)//'.prt')

	write(*,*) 'Kraj'

contains

	subroutine Input(fileName)
	implicit none
	character*(*) fileName
	integer k,icvor,ielem,nRubniUvjet
		open(unit=1,file=fileName,action='READ',status='OLD')
		! ucitavanje cvorova
		read(1,*) Ncvor
		allocate(Xc(Ncvor),Yc(Ncvor))	! alociranje memorije za cvorove
		do k=1,Ncvor
			read(1,*) icvor,Xc(icvor),Yc(icvor)
		enddo
		! ucitavanje elemenata
		read(1,*) Nelem
		allocate(eVeze(3,Nelem))	! alociranje memorije za tablicu veza
		do k=1,Nelem
			read(1,*) ielem,eVeze(1,ielem),eVeze(2,ielem),eVeze(3,ielem)
		enddo
		! ucitavanje rubnih uvjeta
		read(1,*) nRubniUvjet	! broj podataka za rubne uvjete
		! alociranje memorije za rubne uvjete
		allocate(TipRuvjet(Ncvor),RubniUvjet(Ncvor))
		TipRuvjet=0			! inicijalizacija memorije
		RubniUvjet=0		! inicijalizacija memorije
		do k=1,nRubniUvjet
			read(1,*) icvor,TipRuvjet(icvor),RubniUvjet(icvor)
		enddo
		close(1)
	end subroutine Input

	integer function SirinaPojasa()
	implicit none
	integer k,ielem,cvmax,cvmin
		! pretrazi sve elemente i pronadji najvecu
		! razliku indeksa cvorova na elementu
		SirinaPojasa = -Ncvor
		do ielem=1,Nelem
			cvmax=0
			cvmin=Ncvor
			do k=1,3
				cvmax = max(cvmax,eVeze(k,ielem))
				cvmin = min(cvmin,eVeze(k,ielem))
			enddo
			SirinaPojasa = max(SirinaPojasa,(cvmax-cvmin))
		enddo
	end function SirinaPojasa

	subroutine SlaganjeGlobalneMatrice
	implicit none
	integer ielem,r,rr,s,ss,pos
	real Xe(3),Ye(3),Ae(3,3)
		! alociranje globalne memorije
		allocate(Aglob(Ncvor,nStup),Bglob(Ncvor))
		! inicijalizacija
		Aglob=0
		Bglob=0
		! proces asembliranja
		do ielem=1,Nelem
			call ElementniPodaci(ielem,Xe,Ye)
			call Matrica(Xe,Ye,Ae)
			do rr=1,3		! redak u elementnoj matrici
				r = eVeze(rr,ielem)		! redak u globalnoj matrici
				do ss=1,3	! stupac u elementnoj matrici
					s = eVeze(ss,ielem)	! stupac u globalnoj matrici
					pos = s - r + gDiag	! pozicija u pojasnoj matrici
					! superpozicija
					Aglob(r,pos) = Aglob(r,pos)+Ae(rr,ss)
				enddo
			enddo
		enddo
	end subroutine SlaganjeGlobalneMatrice

	subroutine ElementniPodaci(ie,x,y)
	implicit none
	integer ie
	real x(3),y(3)
		! prvi cvor
		x(1)=Xc(eVeze(1,ie))
		y(1)=Yc(eVeze(1,ie))
		! drugi cvor
		x(2)=Xc(eVeze(2,ie))
		y(2)=Yc(eVeze(2,ie))
		! treci cvor
		x(3)=Xc(eVeze(3,ie))
		y(3)=Yc(eVeze(3,ie))
	end subroutine ElementniPodaci

	subroutine Matrica(x,y,elmmtx)
	implicit none
	!
	!     Podprogram za izracunavanje matrice vodljivosti trocvornog
	!     trokutnog konacnog elementa potencijalnog strujanja
	!
	!     Ulazni parametri:
	!     x(3) - x koordinate cvorova trokuta
	!     y(3) - y koordinate cvorova trokuta
	!
	!     Izlazni parametri
	!     elmmtx(3,3) - 3x3 matrica vodljivosti konacnog elementa
	!
		real x(3),y(3)
		real elmmtx(3,3)

		integer i,j		! indeksi, pomocne varijable
		real Det		! determinanta, dvostruka povrsina trokuta
		! pomocna polja za koeficijente koordinata
		real b(3),c(3),d(3)
		! pomocni indeksi za ciklicko izracunavanje koeficijenata b,c,d
		integer ii(3),jj(3),kk(3)
		data ii/1,2,3/, jj/2,3,1/, kk/3,1,2/
	!
		do i=1,3
			b(i) = y(jj(i))-y(kk(i))
			c(i) = x(kk(i))-x(jj(i))
			d(i) = x(jj(i))*y(kk(i))-x(kk(i))*y(jj(i))
		enddo
	!
		Det = d(1)+d(2)+d(3)
	!
		do i=1,3
			do j=1,3
				elmmtx(i,j) = (b(i)*b(j)+c(i)*c(j))/(2.*Det)
			enddo
		enddo
	!
	end subroutine Matrica

	subroutine RubniUvjeti
	implicit none
	integer icvor,idiag,m
		m=(nStup-1)/2+1		!	- pozicija glavne diagonale
		! prisilni rubni uvjeti:
		!	jednostavno se postojeca cvorna jednadzba zamijeni novom
		do icvor = 1,Ncvor
			if(TipRuvjet(icvor).ne.1) cycle
			Bglob(icvor)=RubniUvjet(icvor)
			do idiag = 1,nStup
				Aglob(icvor,idiag)=0.
			enddo
			Aglob(icvor,m)=1.
		enddo
	end subroutine RubniUvjeti

	subroutine Ispis(fileName)
	implicit none
	character*(*) fileName
	integer cv
		open(unit=1,file=fileName)
		do cv=1,Ncvor
			write(1,*) cv,Xc(cv),Yc(cv),Bglob(cv)
		enddo
		close(1)
	end subroutine Ispis

End Program PotencijalnoStrujanje
