Program Filtracija

!   skolski primjer rjesenja stacionarnog procjedjivanja
!   metodom konacnih elemenata
!
!   napisao:
!             Prof. dr. Vinko Jovic
!

! parametri dimenzija modela
integer Ncvor	! broj cvorova
integer Nelem	! broj elemenata
integer Nmat	! broj materijala
integer nStup	! broj stupaca (dijagonala) u matrici u stednom obliku

real,allocatable:: Xc(:)	! alokatibilno polje X koordinata
real,allocatable:: Yc(:)	! alokatibilno polje Y koordinata
real,allocatable:: Darcy(:,:)	! alokatibilno polje koeficijenata filtracije
integer,allocatable:: Materijal(:)	! alokatibilno polje indeks elementnih materijala
integer,allocatable:: eVeze(:,:)	! alokatibilno polje tablicu elementnih veza
!	TipRuvjet(cvor)=0	-	obicni cvor (zbroj tokova jednak nuli)
!	TipRuvjet(cvor)=1	-	zadana vrijednost rjesenja
integer,allocatable:: TipRuvjet(:)	! alokatibilno polje tipa rubnog uvjeta
real,allocatable:: RubniUvjet(:)	! alokatibilno polje vrijednosti rubnog uvjeta

real,allocatable:: Aglob(:,:)	! prostor za globalnu matricu
real,allocatable:: Bglob(:)		! prostor za globalni vektor

character*(256) JobFileName

	write(*,'(A\)') 'Filtracija: Utipkaj ime ulaznog filea:  '
	read(*,*) JobFileName

	call Input(JobFileName)
	nStup=2*SirinaPojasa()-1
	call SlaganjeGlobalneMatrice
	call RubniUvjeti
	call RjesiPojasniSustav(Ncvor,nStup,Aglob,Bglob)
	call Ispis(trim(JobFileName)//'.prt')

	write(*,*) 'Kraj'

contains

	subroutine Input(fileName)
	implicit none
	character*(*) fileName
	integer k,icvor,ielem,nRubniUvjet,imat
		open(unit=1,file=fileName,action='READ',status='OLD')
		! ucitavanje cvorova
		read(1,*) Ncvor
		allocate(Xc(Ncvor),Yc(Ncvor))	! alociranje memorije za cvorove
		do k=1,Ncvor
			read(1,*) icvor,Xc(icvor),Yc(icvor)
		enddo
		! ucitavanje elemenata
		read(1,*) Nelem
		! alociranje memorije za indekse elementnih materijala i tablicu veza
		allocate(Materijal(Nelem),eVeze(3,Nelem))
		do k=1,Nelem
			read(1,*) ielem,Materijal(ielem),eVeze(1,ielem),eVeze(2,ielem),eVeze(3,ielem)
		enddo
		! ucitavanje koeficijenata filtracije Kx i Ky
		read(1,*) Nmat
		allocate(Darcy(2,Nmat))	! alociranje memorije za cvorove
		do k=1,Nmat
			read(1,*) imat,Darcy(1,imat),Darcy(2,imat)
		enddo
		! ucitavanje rubnih uvjeta
		read(1,*) nRubniUvjet	! broj podataka za rubne uvjete
		allocate(TipRuvjet(Ncvor),RubniUvjet(Ncvor))	! alociranje memorije za R.U.
		TipRuvjet=0			! inicijalizacija memorije
		RubniUvjet=0		! inicijalizacija memorije
		do k=1,nRubniUvjet
			read(1,*) icvor,TipRuvjet(icvor),RubniUvjet(icvor)
		enddo
		close(1)
	end subroutine Input

	integer function SirinaPojasa()
	implicit none
	integer k,ielem,cvmax,cvmin
		! pretrazi sve elemente i pronadji najvecu
		! razliku indeksa cvorova na elementu
		SirinaPojasa = -Ncvor
		do ielem=1,Nelem
			cvmax=0
			cvmin=Ncvor
			do k=1,3
				cvmax = max(cvmax,eVeze(k,ielem))
				cvmin = min(cvmin,eVeze(k,ielem))
			enddo
			SirinaPojasa = max(SirinaPojasa,(cvmax-cvmin))
		enddo
		!(Sirina pojasa = glavna + broj diagonala from sa strane)
		SirinaPojasa = SirinaPojasa+1
	end function SirinaPojasa

	subroutine SlaganjeGlobalneMatrice
	implicit none
	integer gDiag,ielem,r,rr,s,ss,pos
	real Xe(3),Ye(3),Ae(3,3),Kx,Ky
		! alociranje globalne memorije
		gDiag=SirinaPojasa()
		allocate(Aglob(Ncvor,2*(gDiag-1)+1),Bglob(Ncvor))
		Aglob=0		! inicijalizacija
		Bglob=0
		! proces asembliranja
		do ielem=1,Nelem
			call ElementniPodaci(ielem,Xe,Ye,Kx,Ky)
			call Matrica(Xe,Ye,Ae,Kx,Ky)
			do rr=1,3		! redak u elementnoj matrici
				r = eVeze(rr,ielem)		! redak u globalnoj matrici
				do ss=1,3	! stupac u elementnoj matrici
					s = eVeze(ss,ielem)	! stupac u globalnoj matrici
					pos = s - r + gDiag	! pozicija u pojasnoj matrici
					! superpozicija
					Aglob(r,pos) = Aglob(r,pos)+Ae(rr,ss)
				enddo
			enddo
		enddo
	end subroutine SlaganjeGlobalneMatrice

	subroutine ElementniPodaci(ie,x,y,Kx,Ky)
	implicit none
	integer ie
	real x(3),y(3),Kx,Ky
		! prvi cvor
		x(1)=Xc(eVeze(1,ie))
		y(1)=Yc(eVeze(1,ie))
		! drugi cvor
		x(2)=Xc(eVeze(2,ie))
		y(2)=Yc(eVeze(2,ie))
		! treci cvor
		x(3)=Xc(eVeze(3,ie))
		y(3)=Yc(eVeze(3,ie))
		Kx=Darcy(1,Materijal(ie))
		Ky=Darcy(2,Materijal(ie))
	end subroutine ElementniPodaci

	subroutine Matrica(x,y,elmmtx,Kx,Ky)
	implicit none
	!
	!     Podprogram za izracunavanje matrice vodljivosti trocvornog
	!     trokutnog konacnog elementa potencijalnog strujanja
	!
	!     Ulazni parametri:
	!     x(3) - x koordinate cvorova trokuta
	!     y(3) - y koordinate cvorova trokuta
	!     Kx - koeficijent filtracije u horizontalnom smjeru
	!     Ky - koeficijent filtracije u uspravnom smjeru
	!
	!     Izlazni parametri
	!     elmmtx(3,3) - 3x3 matrica vodljivosti konacnog elementa
	!
		real x(3),y(3),Kx,Ky
		real elmmtx(3,3)

		integer i,j		! indeksi, pomocne varijable
		real Det		! determinanta, dvostruka povrsina trokuta
		! pomocna polja za koeficijente koordinata
		real b(3),c(3),d(3)
		! pomocni indeksi za ciklicko izracunavanje koeficijenata b,c,d
		integer ii(3),jj(3),kk(3)
		data ii/1,2,3/, jj/2,3,1/, kk/3,1,2/
	!
		do i=1,3
			b(i) = y(jj(i))-y(kk(i))
			c(i) = x(kk(i))-x(jj(i))
			d(i) = x(jj(i))*y(kk(i))-x(kk(i))*y(jj(i))
		enddo
	!
		Det = d(1)+d(2)+d(3)
	!
		do i=1,3
			do j=1,3
				elmmtx(i,j) = (Kx*b(i)*b(j)+Ky*c(i)*c(j))/(2.*Det)
			enddo
		enddo
	!
	end subroutine Matrica

	subroutine RjesiPojasniSustav(nJed,nStup,a,b)
	!
	! Rjesenje nesimetricnog pojasnog sustava linearnih
	! jednadzbi u stednom obliku matrice Gaussovom eliminacijom
	!
	! U pozivu:
	!	nJed - broj jednadzbi
	!	nStup - broj diagonala-stupaca u stednom obliku
	!	A - polje koeficijenata matrice u stednom obliku
	!	B - vektor desne strane
	! U povratku:
	!	B - vektor rjesenja sustava
	!
	!
	implicit none
	integer nJed,nStup
	real a(nJed,nStup),b(nJed)
	real pivot
	integer il,i,j,k,l,m

		m=(nStup-1)/2+1		!	- pozicija glavne diagonale
	!
		do k=1,nJed
			pivot=a(k,m)
			b(k)=b(k)/pivot
			il = k+m-1
			if(il.gt.nJed) il=nJed
			do j=k,il
				l=j-k+m
				a(k,l)=a(k,l)/pivot
			enddo
			if(k.eq.nJed) exit
			do i=k+1,il
				b(i)=b(i)-a(i,k-i+m)*b(k)
				do j=k+1,il
					l=j-i+m
					a(i,l)=a(i,l)-a(i,k-i+m)*a(k,j-k+m)
				enddo
			enddo
		enddo
	!
		do k=nJed-1,1,-1
			il = k+m-1
			if(il.gt.nJed) il=nJed
			do j=k+1,il
				b(k)=b(k)-a(k,j-k+m)*b(j)
			enddo
		enddo
	end subroutine RjesiPojasniSustav

	subroutine RubniUvjeti
	implicit none
	integer icvor,idiag,m
		m=(nStup-1)/2+1		!	- pozicija glavne diagonale
		! prisilni rubni uvjeti:
		!	jednostavno se postojeca cvorna jednadzba zamijeni novom
		do icvor = 1,Ncvor
			if(TipRuvjet(icvor).ne.1) cycle
			Bglob(icvor)=RubniUvjet(icvor)
			do idiag = 1,nStup
				Aglob(icvor,idiag)=0.
			enddo
			Aglob(icvor,m)=1.
		enddo
	end subroutine RubniUvjeti

	subroutine Ispis(fileName)
	implicit none
	character*(*) fileName
	integer cv
		open(unit=1,file=fileName)
		do cv=1,Ncvor
			write(1,*) cv,Xc(cv),Yc(cv),Bglob(cv)
		enddo
		close(1)
	end subroutine Ispis

End Program Filtracija
