Program Putanja
Use ObicneDJ
implicit none

	!parametri i varijable
	real,parameter:: Pi=3.14159
	real,parameter:: g=9.81			! ubrzanje sile teze
	real,parameter:: Ni=1.151e-6	! viskoznost zraka
	real,parameter:: Ro=1.2			! gustoca zraka
	real,parameter:: Rot=145.7		! gustoca lopte (Masa=0.445 kg)
	real,parameter:: D=0.18			! promjer lopte
	real,parameter:: Kut=15			! kut izbacaja lopte
	real,parameter:: V0=20			! brzina izbacaja lopte
	!
	real t				 ! vrijeme
	real Y(4)			 ! polje za zavisne varijable (rjesenje)
	! koristi se equivalence radi preglednosti programa, 
	!varijable rjesenja
	real Vx,Vz		! komponente brzine kugle
	real x,z		! polozaj kugle
	real V,Re,Co	! brzina kugle, Reynoldsov broj i Koeficijent optjecanja
	! vidi napomenu uz potprogram DesnaStrana
	equivalence (x,Y(1))
	equivalence (z,Y(2))
	equivalence (Vx,Y(3))
	equivalence (Vz,Y(4))

	call RungeKuttaPostupak

contains

	subroutine PocetniUvjeti
	implicit none
		! pridruzivanje pocetnih uvjeta
		!---------------
		t=0			! [s]
		x=0
		z=2
		Vx=V0*cos(Pi*Kut/180)
		Vz=V0*sin(Pi*Kut/180)
		!---------------
	end subroutine PocetniUvjeti

	! 'call back' procedura
	subroutine DesnaStrana(n,X,Y,F,ierr)
	! Kako je ova 'call back' procedura unutar glavnog programskog modula
	! u njoj su vidljive sve varijable i konstante deklarirane u glavnom programu.
	! Ova procedura poziva se iz modula ObicneDJ, koji modificira
	! zavisne varijable Y. Stoga se Y prenosi preko parametarske liste
	! pa se u njoj ne smiju koristiti ekvivalentna imena varijabli rjesenja, 
	! jer je equivalence naredba aktivna u glavnom programskom modulu
	! tek nakon povratka iz modula ObicneDJ
	implicit none
	integer n	! broj jednadzbi
	real X		! vrijednost nezavisne varijable
	real Y(n)	! vrijednosti zavisnih varijabli (nepoznanica)
	real F(n)	! vrijednosti desne strane
	integer ierr	! indikator pogreske u proracunu
	! proracun desne strane sustava jednadzbi
		ierr=0	! postaviti >0 ako se javi pogreska u proracunu desne strane
		F(1) = Y(3)	! jednadzba za x
		F(2) = Y(4)	! jednadzba za z
		V=sqrt(Y(3)**2+Y(4)**2)
		if(V.le.0) then
			ierr=1
			return
		else
			Re=V*D/Ni
			if(Re.le.1e4) then
				Co=24*(1+Re**0.66666666/6)/Re
			else
				Co=0.4
			endif
		endif
		! jednadzba za Vx
		F(3) = -6.*Co*Ro*V*Y(3)/(8.*D*Rot)
		! jednadzba za Vz
		F(4) = -g - 6.*Co*Ro*V*Y(4)/(8.*D*Rot)
	end subroutine DesnaStrana

	subroutine RungeKuttaPostupak
	real h/0.01/		! korak integracije u sekundama
	integer Nkorak/1000/	! maksimalni broj koraka
	integer korak		! brojilo koraka
		call PocetniUvjeti
		write(1,*) '  t x z Vx Vz'
		write(1,'(f10.3,4f10.4)') t, x, z, Vx, Vz
		if (OpenObicneDJ(4)) then
			do korak=1,Nkorak
				if (RK_Korak(t,Y,h,DesnaStrana)) then
					write(1,'(f10.3,4f10.4)')  t, x, z, Vx, Vz
					! izlaz iz proracunske petlje kad je z<=0
					if(z.le.0.) exit
				endif
			enddo
			call CloseObicneDJ
		endif
	end subroutine RungeKuttaPostupak

end program Putanja
