	PROGRAM RELAX
C-------------------------------------------------------------------------  		
C   
C     Elementary implementation of
C
C	           Successive Over Relaxation method (SOR)
C     to solve
C     SEEPAGE EQUATION
C
C     by finite difference method, based on finite volume techique
C	in orthogonal raster (mesh)
C
C	Written by
C	             Prof. dr. Vinko Jovic
C
C	                   University of Split
C	                   Faculty of Civil Eng.
C	                   21000 Split, Matice hrvatske b.b.
C	                   Cratia
C
C	Data description
C
C	NTYPE	- array of nodal types in raster (i,j)
C	H		- array of nodal potentials
C	DX,DY	- discretization steps in X and Y coordinate directions
C	EPS		- criteria for convergence
C	KX, KY	- Darcy filtration coeff.
C
C	Notice:
C			Types of nodes are described in text
C------------------------------------------------------------------------
C
	PARAMETER (MXIX=100,MXIY=100)
	INTEGER NTYPE
	REAL H,KX,KY
	COMMON /MESHDT/ H(MXIX,MXIY),NTYPE(MXIX,MXIY),DX,DY,EPS,KX,KY
	DATA KX/1./,KY/1./

C
C	Read input data, see explanation in subroutine
C
	CALL INPUT(M,N,NITER,OMEGA)
C	
C	if data were omitted in input, set default values
	IF(OMEGA.LE.0.) OMEGA=1.
C	IF(NITER.LE.0) NITER=50
	IF(EPS.LE.0.) EPS=0.0001
C
C	Iteration loop in NITER steps
C
	DO 5000 KITER=1,NITER
C
C	initialise norm DHMAX
C
	DHMAX = -1.E34
C
C	scann raster loops
C
	DO 100 J=1,N
	DO 100 I=1,M
C
C	branch to appropriate node type
C	skip nodes with prescribed boundary values an nodes outside flow domain
	IF(NTYPE(I,J).LE.0) GO TO 100
	ITYPE=NTYPE(I,J)
C
	GO TO (1,2,3,4,5,6,7,8,9,10,11,12,13),ITYPE
C
C	Each label is assingned to node type of the same number
C
1	SY=KX*DY/DX+KY*DX/DY+KX*DY/DX+KY*DX/DY
	RES=(H(I+1,J)-H(I,J))*KX*DY/DX+(H(I,J+1)-H(I,J))*KY*DX/DY+
     :	(H(I-1,J)-H(I,J))*KX*DY/DX+(H(I,J-1)-H(I,J))*KY*DX/DY
	GO TO 99
2	SY=0.5*KX*DY/DX+KY*DX/DY+0.5*KX*DY/DX
	RES=(H(I+1,J)-H(I,J))*0.5*KX*DY/DX+(H(I,J+1)-H(I,J))*KY*DX/DY+
     :	(H(I-1,J)-H(I,J))*0.5*KX*DY/DX
	GO TO 99
3	SY=0.5*KX*DY/DX+0.5*KX*DY/DX+KY*DX/DY
	RES=(H(I+1,J)-H(I,J))*0.5*KX*DY/DX+
     :	(H(I-1,J)-H(I,J))*0.5*KX*DY/DX+(H(I,J-1)-H(I,J))*KY*DX/DY
	GO TO 99
4	SY=KX*DY/DX+0.5*KY*DX/DY+0.5*KY*DX/DY
	RES=(H(I+1,J)-H(I,J))*KX*DY/DX+(H(I,J+1)-H(I,J))*0.5*KY*DX/DY+
     :						   +(H(I,J-1)-H(I,J))*0.5*KY*DX/DY
	GO TO 99
5	SY=0.5*KY*DX/DY+KX*DY/DX+0.5*KY*DX/DY
	RES=(H(I,J+1)-H(I,J))*0.5*KY*DX/DY+
     :	(H(I,J-1)-H(I,J))*0.5*KY*DX/DY+(H(I-1,J)-H(I,J))*KX*DY/DX
	GO TO 99
6	SY=0.5*KX*DY/DX+KY*DX/DY+KX*DY/DX+0.5*KY*DX/DY
	RES=(H(I+1,J)-H(I,J))*0.5*KX*DY/DX+(H(I,J+1)-H(I,J))*KY*DX/DY+
     :	(H(I-1,J)-H(I,J))*KX*DY/DX    +(H(I,J-1)-H(I,J))*0.5*KY*DX/DY
	GO TO 99
7	SY=0.5*KX*DY/DX+0.5*KY*DX/DY+KX*DY/DX+KY*DX/DY
	RES=(H(I+1,J)-H(I,J))*0.5*KX*DY/DX+(H(I,J+1)-H(I,J))*0.5*KY*DX/DY+
     :	(H(I-1,J)-H(I,J))*KX*DY/DX    +(H(I,J-1)-H(I,J))*KY*DX/DY
	GO TO 99
8	SY=KX*DY/DX+0.5*KY*DX/DY+0.5*KX*DY/DX+KY*DX/DY
	RES=(H(I+1,J)-H(I,J))*KX*DY/DX    +(H(I,J+1)-H(I,J))*0.5*KY*DX/DY+
     :	(H(I-1,J)-H(I,J))*0.5*KX*DY/DX+(H(I,J-1)-H(I,J))*KY*DX/DY
	GO TO 99
9	SY=KX*DY/DX+KY*DX/DY+0.5*KX*DY/DX+0.5*KY*DX/DY
	RES=(H(I+1,J)-H(I,J))*KX*DY/DX    +(H(I,J+1)-H(I,J))*KY*DX/DY+
     :	(H(I-1,J)-H(I,J))*0.5*KX*DY/DX+(H(I,J-1)-H(I,J))*0.5*KY*DX/DY
	GO TO 99
10	SY=0.5*KX*DY/DX+0.5*KY*DX/DY
	RES=(H(I+1,J)-H(I,J))*0.5*KX*DY/DX+(H(I,J+1)-H(I,J))*0.5*KY*DX/DY
	GO TO 99
11	SY=   0.5*KY*DX/DY+0.5*KX*DY/DX
	RES=(H(I,J+1)-H(I,J))*0.5*KY*DX/DY+(H(I-1,J)-H(I,J))*0.5*KX*DY/DX
	GO TO 99
12	SY=   0.5*KX*DY/DX+0.5*KY*DX/DY
	RES=(H(I-1,J)-H(I,J))*0.5*KX*DY/DX+(H(I,J-1)-H(I,J))*0.5*KY*DX/DY
	GO TO 99
13	SY= 0.5*KX*DY/DX+0.5*KY*DX/DY
	RES=(H(I+1,J)-H(I,J))*0.5*KX*DY/DX+(H(I,J-1)-H(I,J))*0.5*KY*DX/DY
C
99	CONTINUE
C
C	set norm value	
	DH = OMEGA*RES/SY
	DHMAX = AMAX1(DHMAX,ABS(DH))
C	make correction of current nodal potential
	H(I,J)=H(I,J) + DH
100	CONTINUE
C	Check for convergence criteria
	IF(DHMAX.LE.EPS) GO TO 101
5000	CONTINUE
C	
101	CONTINUE
C
C	Output the results
C
	WRITE(2,202) KITER
	DO 52 J=1,N
52	WRITE(2,201) J,(H(I,J),I=1,M)
C	
201	FORMAT(1X,'J=',I3,' I = 1 , M'/10(8F9.3/))
202	FORMAT(1X,'Nodal values of potential. Number of iter.',I5//)

	STOP
	END
C
	SUBROUTINE INPUT(M,N,NITER,OMEGA)
C
	PARAMETER (MXIX=100,MXIY=100)
	INTEGER NTYPE
	REAL H,KX,KY
	COMMON /MESHDT/ H(MXIX,MXIY),NTYPE(MXIX,MXIY),DX,DY,EPS,KX,KY
C
C	read number of division in X and Y direction (i,j direction)
C	read raster step Dx,Dy
C	read Darcy koeff. Kx,Ky
C	read number of iterations NITER
C	read acceleration factor OMEGA
C	read convergence criteria EPS
C
	READ(1,14) M,N,DX,DY,KX,KY,NITER,OMEGA,EPS
C
	WRITE(2,15) M,N,DX,DY,KX,KY,NITER,OMEGA,EPS
C	if omitted set default values
	IF(DX.LE.0.) DX=1.
	IF(DY.LE.0.) DY=1.
	IF(KX.LE.0.) KX=1.
	IF(KY.LE.0.) KY=1.
C
C	set default nodal types
C
	DO 150 I=1,M
	DO 150 J=1,N
150	NTYPE(I,J)=1
	DO 160 I=2,M-1
	NTYPE(I,1)=2
160	NTYPE(I,N)=3
	DO 170 J=2,N-1
	NTYPE(1,J)=4
170	NTYPE(M,J)=5
	NTYPE(1,1)=10
	NTYPE(M,1)=11
	NTYPE(M,N)=12
	NTYPE(1,N)=13
C
C	input and control output of data
C
C
C	read discret coordinates j,i, node type, value of potential
C	apply same type and H value for rectangular part of
C	raster defined by diagonala discret coordinaten
C	(i,j) to (ii,jj)
C	This is a very simple way of data generation!
C
	WRITE(2,17)
C
	DO 5000 KINPUT=1,M*N
	READ(1,200) J,I,ITYPE,HH,JJ,II
	WRITE(2,18) J,I,ITYPE,HH,JJ,II
C
C	data set should be terminated by zero value
	IF(J.LE.0) GO TO 5001
C	do data generation for other nodes
	IF(II.EQ.0) II=I
	IF(JJ.EQ.0) JJ=J
	DO 52 K=I,II
	DO 53 L=J,JJ
	NTYPE(K,L)=ITYPE
53	H(K,L)=HH
52	CONTINUE
C
5000	CONTINUE
C	termination label
5001	CONTINUE
C
C	we do not want start relaxation from zero values
C	but from some mean value, computed from max and min values of
C	prescribed H
C
	HMAX=-1.E34
	HMIN= 1.E34
	DO 24 I=1,M
	DO 24 J=1,N
	IF(NTYPE(I,J).NE.0) GO TO 24
	HMAX=AMAX1(HMAX,H(I,J))
	HMIN=AMIN1(HMIN,H(I,J))
24	CONTINUE
C
	DO 25 I=1,M
	DO 25 J=1,N
	IF(NTYPE(I,J).LE.0) GO TO 25
	H(I,J)=0.5*(HMAX+HMIN)
25	CONTINUE	
C
C	output raster to check nodal types
C	
	WRITE(2,19)
	DO 23  J=1,N
23	WRITE(2,20) (NTYPE(I,J),I=1,M)
C
14	FORMAT(2I5,4F10.0,I6,2F10.0)
15	FORMAT(//'Input control'/' M,N,DX,DY,KX,KY,NITER,OMEGA,EPS'/2I5,
     :2F10.3,2E12.4,I5,F10.5,E12.4)
17	FORMAT(' I,J,NTYPE,H')
18	FORMAT(3I5,E12.4,2I5)
19	FORMAT(/'Nodal types'/' I,J,NTYPE,H'/)
20	FORMAT(1X,100I2)
200	FORMAT(3I5,F10.0,2I5)
30	CONTINUE
C
	RETURN
	END
C
C-----------------------------------------------------------
C
C	Example
C
C	Input data file:	RELAX.INP
C	Output data file:	RELAX.OUT
C