Program ProracunOscilacijaDvijuMasa
Use ObicneDJ
implicit none

	!parametri opruga
    real:: k1 = 1.e5     ! [N/m]
    real:: k2 = 1.e5     ! k2 = k1
	!mase :
    real:: m1 = 1000     ! [kg]
    real:: m2 = 1000     ! m2 = m1
	!podaci za rjesenje
	real t				! vrijeme
	real Y(4)			 ! polje za zavisne varijable (rjesenje)
	real F(4)			 ! polje za desnu stranu jednadzbe

integer postupak/1/		! skretnica za vrstu postupka integracije

	if(postupak.eq.0) call ExplicitniPostupak
	if(postupak.eq.1) call RungeKuttaPostupak
	if(postupak.eq.2) call RungeKuttaFehlbergPostupak
	if(postupak.eq.3) call MjesovitiPostupak

contains
	subroutine PocetniUvjeti
	implicit none
		! pridruzivanje pocetnih uvjeta
		!---------------
		t=0		! [s]
		Y(1)=1		! x1=1  ; [m]
		Y(2)=0		! v1=0   ; [m/s]
		Y(3)=1		! x2=1
		Y(4)=0		! v2=0
		!---------------
	end subroutine PocetniUvjeti

	! 'call back' procedura
	subroutine DesnaStrana(n,X,Y,F,ierr)
	implicit none
	integer n	! broj jednadzbi
	real X		! vrijednost nezavisne varijable
	real Y(n)	! vrijednosti zavisnih varijabli (nepoznanica)
	real F(n)	! vrijednosti desne strane
	integer ierr	! indikator pogreske u proracunu
	! proracun desne strane sustava jednadzbi
		ierr=0	! postaviti >0 ako se javi pogreska u proracunu desne strane
		F(1) = Y(2)
		F(2) = (-(k1+k2) * Y(1) + k2*Y(3))/m1
		F(3) = Y(4)
		F(4) = (k2*Y(1)-k2*Y(3))/m1
	end subroutine DesnaStrana

	subroutine ExplicitniPostupak
	real h/0.001/			! korak integracije
	integer Nkorak/1000/	! maksimalni broj koraka
	integer korak		! brojilo koraka
	integer k			! pomocna varijabla
		call PocetniUvjeti
		write(1,*) '  t x1 v1 x2 v2'
		write(1,'(f10.3,4f10.4)') t,(Y(k),k=1,4)
		if (OpenObicneDJ(4)) then
			do korak=1,Nkorak
				if (Ekspicitni_Korak(t,Y,h,DesnaStrana)) then
					write(1,'(f10.3,4f10.4)') t,(Y(k),k=1,4)
				endif
			enddo
			call CloseObicneDJ
		endif
	end subroutine ExplicitniPostupak

	subroutine MjesovitiPostupak
	real h/0.001/		! korak integracije
	integer Nkorak/1000/	! maksimalni broj koraka
	integer korak		! brojilo koraka
	integer k			! pomocna varijabla
		call PocetniUvjeti
		write(1,*) '  t x1 v1 x2 v2'
		write(1,'(f10.3,4f10.4)') t,(Y(k),k=1,4)
		if (OpenObicneDJ(4)) then
			do korak=1,Nkorak
				if (Mjesoviti_Korak(t,Y,h,0.5,3,DesnaStrana)) then
					write(1,'(f10.3,4f10.4)') t,(Y(k),k=1,4)
				endif
			enddo
			call CloseObicneDJ
		endif
	end subroutine MjesovitiPostupak

	subroutine RungeKuttaPostupak
	real h/0.01/		! korak integracije
	integer Nkorak/100/	! maksimalni broj koraka
	integer korak		! brojilo koraka
	integer k			! pomocna varijabla
		call PocetniUvjeti
		write(1,*) '  t x1 v1 x2 v2'
		write(1,'(f10.3,4f10.4)') t,(Y(k),k=1,4)
		if (OpenObicneDJ(4)) then
			do korak=1,Nkorak
				if (RK_Korak(t,Y,h,DesnaStrana)) then
					write(1,'(f10.3,4f10.4)') t,(Y(k),k=1,4)
				endif
			enddo
			call CloseObicneDJ
		endif
	end subroutine RungeKuttaPostupak

	subroutine RungeKuttaFehlbergPostupak
	real tMax/1./		! kraj vremenskog intervala proracuna
	real h_max/0.5/		! maksimalni
	real h_min/0.00001/	! minimalni korak
	real errors(4)		! polje za prihvat pogresaka u RKF proracunskom koraku
	real s_t,s_Y(4)		! spremanje varijabli (RKF_Step inkrementira varijable)
	integer korak		! brojilo koraka
	real h				! korak integracije
	integer k			! pomocna varijabla
	!external DesnaStrana (aktivirati ako copiler to trazi)
		korak=0
		call PocetniUvjeti
		write(1,*) '  t x1 v1 x2 v2'
		write(1,'(f10.4,4f10.5)') t,(Y(k),k=1,4)
		if (OpenObicneDJ(4)) then
			h=h_max		! pocetni korak
			do while(t<tMax)
				! spremi vrijednosti s pocetka intervala
				s_t=t
				s_Y=Y
				if (RKF_Step(t,Y,h,errors,DesnaStrana)) then
					! ispitivanje tocnosti
					if(abs(errors(1).lt.Toleranca(Y(1),5))) then
						h=h_max		! vrati pocetni korak
						write(1,'(f10.4,4f10.5)') t,(Y(k),k=1,4)
					else
						h=0.5*h	! raspolovi korak
						t=s_t
						Y=s_Y
						if (h<h_min) then
							! odluka o prekidu
							exit	! izlaz iz petlje
						endif
					endif
				else
					! pogreska je nastupila u potprogramu 'DesnaStrana'
					write(1,*) 'Greska u potprogramu "DesnaStrana"'
					! odluka o prekidu
					exit	! izlaz iz petlje
				endif
			enddo
			call CloseObicneDJ
		endif
	end subroutine RungeKuttaFehlbergPostupak

end program
