Program ProracunUspor
use ObicneDJ
implicit none
	! konstante
	real,parameter:: g=9.81		! ubrzanje sile teze
	! podaci
	real:: ho=35		! vodostaj u pocetnom profilu
	real:: Lo=10000		! stacionaza pocetnog profila
	real:: Zo=25		! kota dna pocetnog profila
	real:: Bo=2			! Sirina dna kanala
	real:: Manning=0.02	! n=1/50
	real:: Io=0.001		! Pad dna Io=-dz/dl
	real:: Q=10			! Protok
	! varijable rjesenja u profilu
	real Dubina			! dubina
	real Ie				! pad linije energije
	real z				! kota dna
	real V				! brzina
	real Fr				! Froudeov broj
	real A,O,R			! geometrijski podaci
	!
	! varijable za rjesenje jednadzbe vodnog lica
	!
	!	dy/dl=(Io-Ie)/(1-Fr)
	!
	! za prizmaticko pravokutno korito
	! RUNGE-KUTTA metodom, pomocu modula ObicneDJ
	real l				 ! nezavisna varijabla (stacionaza)
	real Y(1)			 ! polje za zavisne varijable (rjesenje je dubina vode)
	real F(1)			 ! polje za desnu stranu jednadzbe

	equivalence (Dubina,Y(1)) 

	! varijable proracunskih koraka
	real Dl/-500.0/		! razmak profila (predznak '-' za proracun uzvodno)
	integer Nprofila/20/	! maksimalni broj profila
	integer profil		! brojilo profila
	!external DesnaStrana


	call PocetniUvjeti
	
	open(unit=1,File="Uspor.txt")

	write(1,'(f10.2,5f10.4)') l,z,z+Dubina
	if (OpenObicneDJ(1)) then	! jedna diferencijalna jednadzba
		do profil=1,Nprofila
			if (RK_Korak(l,Y,Dl,DesnaStrana)) then
				z=Zo-Io*(l-Lo)
				write(1,'(f10.2,4f10.4)')  l,z,z+Dubina
			endif
		enddo
		call CloseObicneDJ
	endif

	close(1)

contains

	subroutine PocetniUvjeti
	implicit none
		! pridruzivanje pocetnih uvjeta
		! pocetni uvjet je dubina u profilu
		l=Lo			! [m]
		Dubina=(ho-Zo)
		!------parametri presjeka---------
		z=Zo
		A=Bo*Dubina
		O=Bo+2*Dubina
		R=A/O
		V=Q/A
		Ie=V**2/R**(4./3.)*Manning**2
		Fr=V**2/(g*Dubina)
		! normalna dubina Yo=3.706
		! kriticna dubina Yc=1.366
		! kriticni pad Ic=1.112%
		!---------------
	end subroutine PocetniUvjeti

	! 'call back' procedura
	subroutine DesnaStrana(n,X,Y,F,ierr)
	implicit none
	integer n	! broj jednadzbi
	real X		! vrijednost nezavisne varijable
	real Y(n)	! vrijednosti zavisnih varijabli (nepoznanica)
	real F(n)	! vrijednosti desne strane
	integer ierr	! indikator pogreske u proracunu

	! desna strana dif. jednadzbe vodnog lica
	!
	!	dy/dl=(Io-Ie)/(1-Fr)
	!
		A=Bo*Y(1)
		O=Bo+2*Y(1)
		R=A/O
		V=Q/A
		z=Zo-Io*(X-Lo)
		Ie=V**2/R**(4./3.)*Manning**2
		Fr=V**2/(g*Y(1))
		! desna strana
		F(1)=(Io-Ie)/(1.-Fr)

	end subroutine DesnaStrana

end program

