program VodnoLice
!
!-----------------Runge-Kutta-postupak-------------------------
!
use ObicneDJ
use Trapez
implicit none

	! konstante
	integer,parameter:: nZnamenaka=5	! tocnost na broj znamenaka
	! podaci
	real:: Lo=2500		! Duzina poteza trapeznog kanala
	real:: Bo=2			! Sirina dna trapeznog kanala
	real:: m=1			! Nagib pokosa trapeznog kanala
	!real:: Manning=0.02	! hrapavost
	real:: Hrapavost=0.005
	real:: Io=0.001		! Pad dna
	real:: Q=10			! Protok
	real:: Zo=25		! kota dna nizvodnog profila
	! blago nagnuto korito:
	real:: Beta=0.		! kut osi strujanja prema horizontali
	! varijable rjesenja u profilu
	real He				! Energijska kota
	real Hs				! specificna energija
	real Z				! kota dna
	real D				! dubina
	real V				! brzina
	real Fr				! Froudeov broj
	real A,O,R,B		! geometrijski podaci
	real Dn,Dc			! normalna i criticna dubina
	!
	!varijable za rjesenje jednadzbe
	real L				 ! nezavisna varijabla (stacionaza)
	real Y(1)			 ! polje za zavisne varijable (rjesenje He)
	real F(1)			 ! polje za desnu stranu jednadzbe

	equivalence (He,Y(1)) 

	real Dl/50.0/		! razmak profila
	integer Nprofila/50/	! maksimalni broj profila
	integer profil		! brojilo profila
	integer k			! pomocna varijabla
	!external DesnaStrana

	open(unit=1,File="Vodno lice RK.txt")
	write(1,*) '"Stacionaza" "Kota dna" "Vodostaj" "Energija'

	call PocetniUvjeti
	write(1,'(f10.2,5f10.4)') L,Z,Z+D,He
	if (OpenObicneDJ(1)) then	! jedna diferencijalna jednadzba
		do profil=1,Nprofila
			if (RK_Korak(L,Y,Dl,DesnaStrana)) then
				if (ProracunPresjeka(L,He).eq.0) then
					Z=Zo+Io*L
					write(1,'(f10.2,4f10.4)')  L,Z,Z+D,He
				else
					write(*,*) 'Pogreska u proracunu presjeka'
				endif
			endif
		enddo
		call CloseObicneDJ
	endif

	close(1)

contains

	subroutine PocetniUvjeti
	implicit none
	integer ierr	! indikator pogreske u proracunu
	real Ic
		! pridruzivanje pocetnih uvjeta
		!---------------
		L=0			! [m]
		! pocetni uvjet je minimum specificne energije
		D=Dc_Trapez(Q,Bo,m,Beta,nZnamenaka,ierr)
		if(ierr.ne.0) stop 'Pogreska u izracunu kriticne dubine'
		Dc=D
		He=Zo+HS_Trapez(D,Bo,m,Q,Beta,ierr)	! energija u pocetnom profilu
		if(ierr.ne.0) stop 'Pogreska u izracunu spec. energije'
		Z=Zo		! pocetna kota dna
		call AORB_Trapez(Bo,m,D,A,O,R,B)
		V=Q/A
		Dn=DN_Trapez(Q,Hrapavost,Io,Bo,m,nZnamenaka,ierr)
		if(ierr.ne.0) stop 'Pogreska u izracunu normalne dubine'
		!---------------
		Ic=IC_trapez(Q,Bo,m,Hrapavost,nZnamenaka,ierr)
	end subroutine PocetniUvjeti

	! 'call back' procedura
	subroutine DesnaStrana(n,X,Y,F,ierr)
	implicit none
	integer n	! broj jednadzbi
	real X		! vrijednost nezavisne varijable
	real Y(n)	! vrijednosti zavisnih varijabli (nepoznanica)
	real F(n)	! vrijednosti desne strane
	integer ierr	! indikator pogreske u proracunu

		! proracun presjeka (vraca kontrolu pogreske)
		ierr=ProracunPresjeka(X,Y(1))
		if (ierr>0) return
		! desna strana je (+-) Ie=Lambda|V|V/(8gR)
		! (Predznak je + za proracun mirnog rezima uzvodno)
		F(1)=Ie_trapez(Q,D,Bo,m,Hrapavost)

	end subroutine DesnaStrana

	integer function ProracunPresjeka(Stacionaza,Henergije)
	implicit none
	real Stacionaza,Henergije
	real zPriv	! pomocna varijabla
	integer ierr	! indikator pogreske u proracunu
		! izracunaj kotu dna profila
		zPriv=Zo+Io*Stacionaza
		! izracunaj dubinu u mirnom rezimu
		Hs=Henergije-zPriv
		D=DM_Trapez(Q,Hs,Bo,m,Beta,nZnamenaka,ierr)
		if(ierr.ne.0) then
			ProracunPresjeka=ierr	! vraca kontrolu pogreske
		else
			call AORB_Trapez(Bo,m,D,A,O,R,B)
			V=Q/A
			ProracunPresjeka=0	! vraca kontrolu pogreske
		endif
	end function ProracunPresjeka

end program